let shapes = [];
let numShapes = 20;

function setup() {
    createCanvas(800, 600);
    for (let i = 0; i < numShapes; i++) {
        let shapeType = random(['circle', 'square', 'triangle']);
        let x = random(width);
        let y = random(height);
        let size = random(30, 70);
        let colorValue = color(random(255), random(255), random(255));
        shapes.push({ shapeType, x, y, size, colorValue });
    }
}

function draw() {
    background(220);

    for (let shape of shapes) {
        fill(shape.colorValue);
        noStroke();

        if (shape.shapeType === 'circle') {
            ellipse(shape.x, shape.y, shape.size);
        } else if (shape.shapeType === 'square') {
            rect(shape.x, shape.y, shape.size, shape.size);
        } else if (shape.shapeType === 'triangle') {
            triangle(
                shape.x, shape.y - shape.size / 2,
                shape.x - shape.size / 2, shape.y + shape.size / 2,
                shape.x + shape.size / 2, shape.y + shape.size / 2
            );
        }
    }
    
    // Optional: Animate the shapes
    for (let shape of shapes) {
        shape.x += random(-1, 1);
        shape.y += random(-1, 1);
    }
}

function mousePressed() {
    for (let shape of shapes) {
        let d;
        if (shape.shapeType === 'circle') {
            d = dist(mouseX, mouseY, shape.x, shape.y);
            if (d < shape.size / 2) {
                shape.colorValue = color(random(255), random(255), random(255));
                shape.size = random(30, 70);  // Change size on click
            }
        } else if (shape.shapeType === 'square') {
            d = mouseX > shape.x && mouseX < shape.x + shape.size &&
                mouseY > shape.y && mouseY < shape.y + shape.size;
            if (d) {
                shape.colorValue = color(random(255), random(255), random(255));
                shape.size = random(30, 70);
            }
        } else if (shape.shapeType === 'triangle') {
            if (isPointInTriangle(mouseX, mouseY, shape.x, shape.y, 
                shape.x - shape.size / 2, shape.y + shape.size / 2, 
                shape.x + shape.size / 2, shape.y + shape.size / 2)) {
                shape.colorValue = color(random(255), random(255), random(255));
                shape.size = random(30, 70);
            }
        }
    }
}

function isPointInTriangle(px, py, x1, y1, x2, y2, x3, y3) {
    let area = 0.5 * (-y2 * x1 + y1 * (-x2 + x3) + x2 * y1 + y1 * x3);
    let sign = area < 0 ? -1 : 1;
    let s = (y1 * x3 - x1 * y3 + (y3 - y1) * px + (x1 - x3) * py) * sign;
    let t = (x1 * y2 - y1 * x2 + (y1 - y2) * px + (x2 - x1) * py) * sign;

    return s >= 0 && t >= 0 && (s + t) <= 2 * area * sign;
}